@@ -519,14 +519,14 @@ panel.bwplot <-
 
         ## box
 
-        panel.rect(xleft = blist.stats[, 2],
-                   xright = blist.stats[, 4],
-                   y = levels.fos, 
-                   height = blist.height,
-                   lwd = box.rectangle$lwd,
-                   lty = box.rectangle$lty,
-                   fill = fill,
-                   col = box.rectangle$col)
+        lrect(xleft = blist.stats[, 2],
+              xright = blist.stats[, 4],
+              y = levels.fos, 
+              height = blist.height,
+              lwd = box.rectangle$lwd,
+              lty = box.rectangle$lty,
+              col = fill,
+              border = box.rectangle$col)
 
         ## whiskers
 
@@ -598,14 +598,14 @@ panel.bwplot <-
 
         ## box
 
-        panel.rect(ybottom = blist.stats[, 2],
-                   ytop = blist.stats[, 4],
-                   x = levels.fos, 
-                   width = blist.height,
-                   lwd = box.rectangle$lwd,
-                   lty = box.rectangle$lty,
-                   fill = fill,
-                   col = box.rectangle$col)
+        lrect(ybottom = blist.stats[, 2],
+              ytop = blist.stats[, 4],
+              x = levels.fos, 
+              width = blist.height,
+              lwd = box.rectangle$lwd,
+              lty = box.rectangle$lty,
+              col = fill,
+              border = box.rectangle$col)
 
         ## whiskers
 
@@ -1098,7 +1098,21 @@ panel.violin <-
 ### dotplot, barchart and stripplot: essentially wrappers to bwplot
 
 
-dotplot <- function(formula, ...) UseMethod("dotplot")
+
+dotplot <- function(x, ...)
+{
+    ocall <- match.call()
+    formula <- ocall$formula
+    if (!is.null(formula))
+    {
+        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+        ocall$formula <- NULL
+        if (!("x" %in% names(ocall))) ocall$x <- formula else warning("'formula' overridden by 'x'")
+        eval(ocall, parent.frame())
+    }
+    else UseMethod("dotplot")
+}
+
 
 
 ## dotplot.numeric <-
@@ -1121,14 +1135,14 @@ dotplot <- function(formula, ...) UseMethod("dotplot")
 ## }
 
 dotplot.numeric <-
-    function(formula, data = NULL, xlab = deparse(substitute(formula)), ...)
+    function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
     ocall <- ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit data specification ignored")
-    ccall$data <- list(x = formula)
+    ccall$data <- list(x = x)
     ccall$xlab <- xlab
-    ccall$formula <- ~x
+    ccall$x <- ~x
     ccall[[1]] <- as.name("dotplot")
     ans <- eval(ccall, parent.frame())
     ans$call <- ocall
@@ -1144,8 +1158,9 @@ dotplot.numeric <-
 
 
 dotplot.table <-
-    function(formula, data = NULL, groups = TRUE, ...)
+    function(x, data = NULL, groups = TRUE, ...)
 {
+    formula <- x
     ocall <- match.call()
     ## formula <- eval(substitute(~foo, list(foo = substitute(formula))))
     data <- as.data.frame(formula)
@@ -1175,13 +1190,13 @@ dotplot.table <-
 }
 
 
-dotplot.default <- function(formula, ...) dotplot(table(formula), ...)
-dotplot.array <- function(formula, ...) dotplot(as.table(formula), ...)
-dotplot.matrix <- function(formula, ...) dotplot(as.table(formula), ...)
+dotplot.default <- function(x, ...) dotplot(table(x), ...)
+dotplot.array <- function(x, ...) dotplot(as.table(x), ...)
+dotplot.matrix <- function(x, ...) dotplot(as.table(x), ...)
 
 
 dotplot.formula <-
-    function(formula,
+    function(x,
              data = parent.frame(),
              panel = "panel.dotplot",
              ...)
@@ -1196,18 +1211,30 @@ dotplot.formula <-
 }
 
 
-barchart <- function(formula, ...) UseMethod("barchart")
+barchart <- function(x, ...)
+{
+    ocall <- match.call()
+    formula <- ocall$formula
+    if (!is.null(formula))
+    {
+        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+        ocall$formula <- NULL
+        if (!("x" %in% names(ocall))) ocall$x <- formula else warning("'formula' overridden by 'x'")
+        eval(ocall, parent.frame())
+    }
+    else UseMethod("barchart")
+}
 
 
 barchart.numeric <-
-    function(formula, data = NULL, xlab = deparse(substitute(formula)), ...)
+    function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
     ocall <- ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit data specification ignored")
-    ccall$data <- list(x = formula)
+    ccall$data <- list(x = x)
     ccall$xlab <- xlab
-    ccall$formula <- ~x
+    ccall$x <- ~x
     ccall[[1]] <- as.name("barchart")
     ans <- eval(ccall, parent.frame())
     ans$call <- ocall
@@ -1218,9 +1245,10 @@ barchart.numeric <-
 
 
 barchart.table <-
-    function(formula, data = NULL, groups = TRUE,
+    function(x, data = NULL, groups = TRUE,
              origin = 0, stack = TRUE, ...)
 {
+    formula <- x
     ocall <- match.call()
     if (!is.null(data)) warning("explicit data specification ignored")
     ## formula <- eval(substitute(~foo, list(foo = substitute(formula))))
@@ -1244,20 +1272,22 @@ barchart.table <-
         form <- paste(form, rest, sep = "|")
     }
     ans <-
-        barchart(as.formula(form), data, groups = eval(groups),
+        barchart(as.formula(form), data,
+                 groups = eval(groups),
+                 ##groups = groups,
                  origin = origin, stack = stack, 
                  ...)
     ans$call <- ocall
     ans
 }
 
-barchart.default <- function(formula, ...) barchart(table(formula), ...)
-barchart.array <- function(formula, ...) barchart(as.table(formula), ...)
-barchart.matrix <- function(formula, ...) barchart(as.table(formula), ...)
+barchart.default <- function(x, ...) barchart(table(x), ...)
+barchart.array <- function(x, ...) barchart(as.table(x), ...)
+barchart.matrix <- function(x, ...) barchart(as.table(x), ...)
 
 
 barchart.formula <-
-    function(formula,
+    function(x,
              data = parent.frame(),
              panel = "panel.barchart",
              box.ratio = 2, 
@@ -1274,18 +1304,30 @@ barchart.formula <-
 }
 
 
-stripplot <- function(formula, ...) UseMethod("stripplot")
+stripplot <- function(x, ...)
+{
+    ocall <- match.call()
+    formula <- ocall$formula
+    if (!is.null(formula))
+    {
+        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+        ocall$formula <- NULL
+        if (!("x" %in% names(ocall))) ocall$x <- formula else warning("'formula' overridden by 'x'")
+        eval(ocall, parent.frame())
+    }
+    else UseMethod("stripplot")
+}
 
 
 stripplot.numeric <-
-    function(formula, data = NULL, xlab = deparse(substitute(formula)), ...)
+    function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
     ocall <- ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit data specification ignored")
-    ccall$data <- list(x = formula)
+    ccall$data <- list(x = x)
     ccall$xlab <- xlab
-    ccall$formula <- ~x
+    ccall$x <- ~x
     ccall[[1]] <- as.name("stripplot")
     ans <- eval(ccall, parent.frame())
     ans$call <- ocall
@@ -1295,7 +1337,7 @@ stripplot.numeric <-
 
 
 stripplot.formula <-
-    function(formula,
+    function(x,
              data = parent.frame(),
              panel = "panel.stripplot",
              ...)
@@ -1314,19 +1356,31 @@ stripplot.formula <-
 
 ### bwplot (the workhorse)
 
-bwplot <- function(formula, ...) UseMethod("bwplot")
+bwplot <- function(x, ...)
+{
+    ocall <- match.call()
+    formula <- ocall$formula
+    if (!is.null(formula))
+    {
+        warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
+        ocall$formula <- NULL
+        if (!("x" %in% names(ocall))) ocall$x <- formula else warning("'formula' overridden by 'x'")
+        eval(ocall, parent.frame())
+    }
+    else UseMethod("bwplot")
+}
 
 
 
 bwplot.numeric <-
-    function(formula, data = NULL, xlab = deparse(substitute(formula)), ...)
+    function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
     ocall <- ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit data specification ignored")
-    ccall$data <- list(x = formula)
+    ccall$data <- list(x = x)
     ccall$xlab <- xlab
-    ccall$formula <- ~x
+    ccall$x <- ~x
     ccall[[1]] <- as.name("bwplot")
     ans <- eval(ccall, parent.frame())
     ans$call <- ocall
@@ -1337,7 +1391,7 @@ bwplot.numeric <-
 
 
 bwplot.formula <-
-    function(formula,
+    function(x,
              data = parent.frame(),
              allow.multiple = is.null(groups) || outer,
              outer = FALSE,
@@ -1362,6 +1416,7 @@ bwplot.formula <-
              subscripts = !is.null(groups),
              subset = TRUE)
 {
+    formula <- x
     dots <- list(...)
     groups <- eval(substitute(groups), data, parent.frame())
     subset <- eval(substitute(subset), data, parent.frame())
@@ -1563,7 +1618,7 @@ bwplot.formula <-
     cond.current.level <- rep(1, number.of.cond)
 
 
-    for (panel.number in seq(length = nplots))
+    for (packet.number in seq(length = nplots))
     {
 
         id <- !id.na
@@ -1583,12 +1638,12 @@ bwplot.formula <-
         {
             if (is.f.y)
             {
-                foo$panel.args[[panel.number]] <-
+                foo$panel.args[[packet.number]] <-
                     list(x = x[id],
                          ##y = as.numeric(y[id]))
                          y = y[id])
                 if (subscripts)
-                    foo$panel.args[[panel.number]]$subscripts <-
+                    foo$panel.args[[packet.number]]$subscripts <-
                         subscr[id]
             }
             else  # shingle
@@ -1603,11 +1658,11 @@ bwplot.formula <-
                     panel.y <- c(panel.y, rep(k,length(tid[tid])))
                     if (subscripts) panel.subscr <- c(panel.subscr, subscr[tid])
                 }
-                foo$panel.args[[panel.number]] <-
+                foo$panel.args[[packet.number]] <-
                     list(x = panel.x,
                          y = panel.y)
                 if (subscripts)
-                    foo$panel.args[[panel.number]]$subscripts <-
+                    foo$panel.args[[packet.number]]$subscripts <-
                         panel.subscr
 
             }
@@ -1616,12 +1671,12 @@ bwplot.formula <-
         {
             if (is.f.x)
             {
-                foo$panel.args[[panel.number]] <-
+                foo$panel.args[[packet.number]] <-
                     ##list(x = as.numeric(x[id]),
                     list(x = x[id],
                          y = y[id])
                 if (subscripts)
-                    foo$panel.args[[panel.number]]$subscripts <-
+                    foo$panel.args[[packet.number]]$subscripts <-
                         subscr[id]
             }
             else   # shingle
@@ -1636,11 +1691,11 @@ bwplot.formula <-
                     panel.x <- c(panel.x, rep(k,length(tid[tid])))
                     if (subscripts) panel.subscr <- c(panel.subscr, subscr[tid])
                 }
-                foo$panel.args[[panel.number]] <-
+                foo$panel.args[[packet.number]] <-
                     list(x = panel.x,
                          y = panel.y)
                 if (subscripts)
-                    foo$panel.args[[panel.number]]$subscripts <-
+                    foo$panel.args[[packet.number]]$subscripts <-
                         panel.subscr
             }
         }
