@@ -159,7 +159,7 @@ xyplot <- function(x, ...)
     {
         warning("The 'formula' argument has been renamed to 'x'. See ?xyplot")
         ocall$formula <- NULL
-        if (is.null(ocall$x)) ocall$x <- formula
+        if (!("x" %in% names(ocall))) ocall$x <- formula else warning("'formula' overridden by 'x'")
         eval(ocall, parent.frame())
     }
     else UseMethod("xyplot")
